;;; These are my c/c++ misc functions. The code for most of these functions
;;; has been borrowed from other peoples .emacs files.

;;; LAST MODIFIED ON:  Thu 25-Jan-2001 13:19:44
;;; LAST MODIFIED BY:  Javier Oviedo


;;;
(defvar c++-header-ext-regexp "\\.\\(hpp\\|h\\|\hh\\|H\\)$")
(defvar c++-source-ext-regexp "\\.\\(cpp\\|c\\|\cc\\|C\\)$")

(defvar c++-source-extension-list '("c" "cc" "C" "cpp" "cxx" "c++"))
(defvar c++-header-extension-list '("h" "hh" "H" "hpp" "hxx" "h++"))
;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Activates hungry-delete for one backspace and then deactivates it.
(defun my-hungry-delete ()
  (interactive)
  (c-toggle-hungry-state 1)
  (call-interactively 'c-electric-backspace)
  (c-toggle-hungry-state -1))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Updates the timpe stamp of the current file.
(defun my-modification-stamp ()
  (interactive)
  (save-excursion
    (if (eq major-mode 'c-mode)
	(insert
	 "/**********************************************************************\n\n"
	 "**********************************************************************/\n")
      (insert
       "/*---------------------------------------------------------------------\n\n"
       "---------------------------------------------------------------------*/\n"))
    (backward-sentence 2)
    (forward-line 1)
    (insert "LAST MODIFIED ON:  \nLAST MODIFIED BY:  Javier Oviedo")
    (save-buffer)))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Insert my standard c/c++ header in the current buffer
(defun my-c-header (course proj due pur)
  (interactive "sCourse Name: \nsProject #: \nsDue Date: \nsPurpose: ")
  (beginning-of-buffer)
  (if (eq major-mode 'c-mode)
      (insert
       "/*********************************************************************/\n"
       "/*********************************************************************/\n"
       "/*******  Javier Oviedo  *********************************************/\n"
       "/*******\n\n"
       "*******/\n"
       "/*********************************************************************/\n"
       "/*********************************************************************/\n\n")
    
    (insert
     "/*-------------------------------------------------------------------*/\n"
     "/*-------------------------------------------------------------------*/\n"
     "/*------  Javier Oviedo  --------------------------------------------*/\n"
     "/*------\n\n"
     "------*/\n"
     "/*-------------------------------------------------------------------*/\n"
     "/*-------------------------------------------------------------------*/\n\n"
))
  (setq name (buffer-name))
  (backward-word 1)
  (forward-line 2)
  (insert "\nCourse Name:  " course "\n")
  (insert "Project #:    " proj "\n")
  (insert "File Name:    " name "\n")
  (insert "Due Date:     " due "\n\n")
  (insert "Purpose: \n")
  (let ((start (point)))
    (insert "         " pur)
    (fill-region start (point)))
  (forward-line 5)
  (save-excursion
    (call-interactively 'my-c-define))
  (forward-line 3)
  (if (eq major-mode 'c-mode)
      (insert "#include <stdio.h>\n\n\n")
    (insert "#include <iostream.h>\n\n\n"))
  (save-buffer))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Inserts ifndef/define/endif for cc-mode header files.
(defun my-c-define ( buf )
  (interactive "b")
  (save-excursion
    (set-buffer buf)
    (if (string-match c++-header-ext-regexp (buffer-name))
	(let ((bufname (buffer-name))
	      defname
	      defstringif
	      defstringend)
	  (setq defname (concat (upcase (file-name-sans-extension bufname)) "_"
				(upcase (file-name-extension bufname))))
	  (setq defstringif (concat
			     "#ifndef " defname "\n"
			     "#define " defname "\n\n"))
	  (setq defstringend (concat
			      "\n\n" "#endif /* " defname " */"))
	  (insert defstringif)
	  (end-of-buffer)
	  (insert defstringend)
	  (save-buffer)))))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Inserts a comment header for a c function definition.
(defun function-header (pur ret)
  "Inserts a comment header for a function definition."
  (interactive "sPurpose: \nsReturns: ")
  (insert
   "/*********************************************************************/\n"
   "/**\n")

  (insert "Purpose: \n")
  (let ((start (point)))
    (insert "   " pur "\n\n")
    (fill-region start (point)))

  (insert "Returns: \n")
  (let ((start (point)))
    (insert "   " ret "\n")
    (fill-region start (point)))

  (insert
   "**/\n"
   "/*********************************************************************/\n"
   "\n"))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Insert a comment header for a c++ in-class method.
(defun method-header (pur ret)
  (interactive "sPurpose: \nsReturns: ")
  (insert
   "/*-------------------------------------------------------------------*/\n"
   "/*-\n")
  
  (insert "Purpose: \n")
  (let ((start (point)))
    (insert "   " pur "\n\n")
    (fill-region start (point)))
  
  (insert "Returns: \n")
  (let ((start (point)))
    (insert "   " ret "\n")
    (fill-region start (point)))
  
  (insert
   "-*/\n"
   "/*-------------------------------------------------------------------*/\n"
   "\n"))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Insert a comment header for a c++ class definition.
(defun class-header (name pur)
  (interactive "sClass Name: \nsPurpose: ")
  (insert
   "/*-------------------------------------------------------------------*/\n"
   "/*-------------------------------------------------------------------*/\n"
   "/*--\n")
  
  (insert "Class Name:  " name "\n\n")
  (insert "Purpose: \n" "   ")
  (let ((start (point)))
    (insert pur "\n")
    (fill-region start (point)))
  
  (insert
   "--*/\n"
   "/*-------------------------------------------------------------------*/\n"
   "/*-------------------------------------------------------------------*/\n"
   "\n\n")
  (save-buffer))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; If point is in a class definition, return the name of the class.
;; Otherwise, return nil.
(defun get-classname ()
  "If the point is in a class definition, get the name of the class.  Return
nil otherwise."
  (save-excursion
    (let ((brace (assoc 'inclass (c-guess-basic-syntax))))
      (if (null brace) '()
        (goto-char (cdr brace))
        (let ((class-open (assoc 'class-open (c-guess-basic-syntax))))
          (if class-open (goto-char (cdr class-open)))
          (if (looking-at "^class[ \t]+\\([A-Za-z_][^ \t:{]*\\)")
              (buffer-substring (match-beginning 1) (match-end 1))
            (error "Error parsing class definition!")))))))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Insert a c++ class definition.
(defun new-class (name)
  "Insert a c++ class definition."
  (interactive "sClass Name: ")
  (insert "class " name "\n{\n   public:\n\n   private:\n};\n")
  (save-buffer))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Insert method prototype in current header file and matching method body
;; in implementation file. 
(defun new-method (rettype proto)
  "Insert a method into the current header file at point. Insert
implementation, too.  This function expects the implementation file to
be named foo.cpp and in the same directory as the current file, foo.h."
  (interactive "sReturn type: \nsPrototype: ")
  (let ((classname (get-classname))
        (c-tab-always-indent t))
    (if (null classname) (message "Not in class definition!")
      (unless (string-equal rettype "") (setq rettype (concat rettype " ")))
      (insert rettype proto ";\n")
      (c-indent-command)
      (save-window-excursion
        (find-file (concat
		    (file-name-sans-extension (buffer-file-name)) ".cpp"))
        (end-of-buffer)
        (insert "\n\n\n")
	(call-interactively 'method-header)
	(insert rettype classname)
	(c-toggle-hungry-state t)
	(delete-backward-char 1)
	(c-toggle-hungry-state nil)
	(insert "::" proto"\n{\n\n}\n")
	(save-buffer))
      (save-buffer))))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Insert a function prototype in current header file and matching
;; function body in source file.
;; (defun new-function (rettype proto name)
;;   "Insert a function into the current header file at point. Insert
;; implementation, too.  This function can take the source file to be the .c
;; version of the header or any other source file that the user wants to
;; specify. Files must be in the same directory, though."
;;   (interactive "sReturn type: \nsPrototype: \nsSource file: ")
;;   (setq c-tab-always-indent t)
;;   (insert rettype " " proto ";\n")
;;   (c-indent-command)
;;   (save-window-excursion
;;     (if (string-equal name "")
;; 	(find-file (concat
;; 		    (file-name-sans-extension (buffer-file-name)) ".c"))
;;       (find-file name))
;;     (end-of-buffer)
;;     (insert "\n\n\n")
;;     (call-interactively 'function-header)
;;     (insert rettype " " proto"\n{\n\n}\n")
;;     (insert "Name: " name "\n")
;;     (save-buffer))
;;   (save-buffer))
(defun new-function (rettype funcname param sourcename)
  "Insert a function into the current header file at point. Insert
implementation, too.  This function can take the source file to be the .c
version of the header or any other source file that the user wants to
specify. Files must be in the same directory, though."
  (interactive "sReturn type: \nsFunction name: \nsParameters: \nsSource file: ")
  (setq c-tab-always-indent t)
  (insert rettype " " funcname "(" param  ");\n")
  (c-indent-command)
  (save-window-excursion
    (if (string-equal sourcename "")
	(find-file (concat
		    (file-name-sans-extension (buffer-file-name)) ".c"))
      (find-file sourcename))
    (end-of-buffer)
    (insert "\n\n\n")
    (call-interactively 'function-header)
    (insert rettype " " funcname "(" param  ")\n{\n\n}\n")
    (backward-sentence 4)
    (forward-line 2)
    (insert "Function Name: " funcname "\n\n")
    (save-buffer))
  (save-buffer))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Insert flagged debug statement
(defun debug-clause (output)
  (interactive "sDebug statement to output: \n")
  "Insert a fresh debugging printf() or cout -statement and position point in it"
  (c-indent-command)
  (insert-string "/* DEBUG */\n")
  (c-indent-command)
  (if (string-equal output "")
      (setq output "I AM TRYING TO DEBUG!!!"))
  (if (eq major-mode 'c-mode)
      (let ()
	(insert-string "printf(\"")
	(insert output)
	(insert-string "\\n\");\n"))
    (let ()
      (insert-string "cout << \"")
      (insert output)
      (insert-string "\" <<endl;\n")))
;;   (if (eq major-mode 'c-mode)
;;       (insert-string "printf(\"I AM TRYING TO DEBUG!!!\\n\");\n")
;;     (insert-string "cout << \"I AM TRYING TO DEBUG!!!\" <<endl;\n"))
  (c-indent-command)
  (insert-string "/* DEBUG */\n")
  (c-indent-command)
  (previous-line 2)
  (forward-word 3)
  (forward-char 3)
  (save-buffer))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Remove flagged debug statements
(defun delete-debugs ()
  (interactive)
  "eliminate sections bounded by /* DEBUG */ <stuff> /* DEBUG */"
  (save-excursion
    (goto-char (point-min))
    (query-replace-regexp 
     "[\t\n ]*/\\* DEBUG \\*/[^/]+/\\* DEBUG \\*/[\t\n ]*\n" 
     "\n\n"))
  (save-buffer))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Switches between source/header files
(defun my-toggle-source-header()
  "Switches to the source buffer if currently in the header buffer and vice versa."
  (interactive)
  (let ((buf (current-buffer))
        (name (file-name-nondirectory (buffer-file-name)))
        file
        offs)
    (setq offs (string-match c++-header-ext-regexp name))
    (if offs
        (let ((lst c++-source-extension-list)
              (ok nil)
              ext)
          (setq file (substring name 0 offs))
          (while (and lst (not ok))
            (setq ext (car lst))
            (if (file-exists-p (concat file "." ext))
		(setq ok t))
            (setq lst (cdr lst)))
          (if ok
              (find-file (concat file "." ext))))
      (let ()
        (setq offs (string-match c++-source-ext-regexp name))
        (if offs
            (let ((lst c++-header-extension-list)
                  (ok nil)
                  ext)
              (setq file (substring name 0 offs))
              (while (and lst (not ok))
                (setq ext (car lst))
                (if (file-exists-p (concat file "." ext))
                    (setq ok t))
                (setq lst (cdr lst)))
              (if ok
                  (find-file (concat file "." ext)))))))))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Jumps to the next slot of the current expansion. It then indents according
;; to mode. 
(defun my-expand-jump-to-next-slot ()
  "Jumps to the next slot of the current expansion. It then indents according to mode"
  (interactive)
  (expand-jump-to-next-slot)
  (indent-according-to-mode))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Jumps to the previous slot of the current expansion. It then indents
;; according to mode.
(defun my-expand-jump-to-previous-slot ()
  "Jumps to the previous slot of the current expansion. It then indents according to mode"
  (interactive)
  (expand-jump-to-previous-slot)
  (indent-according-to-mode))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(provide 'misc-c-functions)
