;; Posted to comp.emacs by
;; Kevin Rodgers <kevinr@ihs.com>          Lead Software Engineer
;;
(defvar line-number-total nil
  "The total number of lines in this buffer.")
(make-variable-buffer-local 'line-number-total)

(defun set-line-number-total (&optional buffer)
  "Set line-number-total to the total number of lines in this buffer."
  (save-excursion
    (set-buffer (or buffer (current-buffer)))
    (setq line-number-total
          (let ((lines (count-lines (point-min) (point-max)))
                (last-char (char-after (1- (point-max)))))
            (format "%d" (if (or (null last-char)
                                 (equal last-char ?\n))
                             (1+ lines)
                           lines))))))

(defadvice display-buffer (after set-line-number-total activate)
  (set-line-number-total (ad-get-arg 0))) ; BUFFER

(defun initialize-line-number-total (buffer)
  (or line-number-total
      (set-line-number-total buffer)))

(defadvice get-buffer-create (after set-line-number-total activate)
  (initialize-line-number-total (get-buffer (ad-get-arg 0)))) ; BUFFER

(let* ((line-number-mode-format
        (assq 'line-number-mode
              (setq-default mode-line-format
                            (copy-alist (default-value 'mode-line-format)))))
       (format (car (cdr line-number-mode-format)))
       (percent-l (string-match "%l" format)))
  (setcdr line-number-mode-format
          (list (list (substring format 0 (+ percent-l 2))
                      (list 'line-number-total
                            (list "/" 'line-number-total))
                      (substring format (+ percent-l 2))))))

(add-hook 'post-command-hook 'set-line-number-total)

(provide 'my-line-number)

